@@ -59,6 +59,32 @@ void pkg_hash_deinit(hash_table_t *hash)
 }
 
 
+/* Find the default arch for a given package status file if none is given. */
+static char *pkg_get_default_arch(ipkg_conf_t *conf)
+{
+     nv_pair_list_elt_t *l;
+     char *def_arch = HOST_CPU_STR;		/* Default arch */
+     int def_prio = 0;				/* Other archs override this */
+
+     l = conf->arch_list.head;
+
+     while (l) {
+	  nv_pair_t *nv = l->data;
+	  int priority = strtol(nv->value, NULL, 0);
+
+	  /* Check if this arch has higher priority, and is valid */
+	  if ((priority > def_prio) &&
+	      (strcmp(nv->name, "all")) && (strcmp(nv->name, "noarch"))) {
+	       /* Our new default */
+	       def_prio = priority;
+	       def_arch = nv->name;
+	  }
+	  l = l->next;
+     }
+
+     return strdup(def_arch);
+}
+
 int pkg_hash_add_from_file(ipkg_conf_t *conf, const char *file_name,
 			   pkg_src_t *src, pkg_dest_t *dest, int is_status_file)
 {
@@ -77,12 +103,12 @@ int pkg_hash_add_from_file(ipkg_conf_t *conf, const char *file_name,
 	  if (pkg_parse_raw(pkg, &raw, src, dest) == 0) {
 	       if (!pkg->architecture) {
 		    char *version_str = pkg_version_str_alloc(pkg);
-		    pkg->architecture = strdup(HOST_CPU_STR);
+		    pkg->architecture = pkg_get_default_arch(conf);
 		    ipkg_message(conf, IPKG_ERROR, "Package %s version %s has no architecture specified, defaulting to %s.\n",
 				 pkg->name, version_str, pkg->architecture);
 		    free(version_str);
 	       }
-	       hash_insert_pkg(hash, pkg, is_status_file);
+	       hash_insert_pkg(hash, pkg, is_status_file,conf);
 	  } else {
 	       free(pkg);
 	  }
@@ -113,7 +139,7 @@ abstract_pkg_vec_t *pkg_hash_fetch_all_installation_candidates(hash_table_t *has
 
 
 pkg_t *pkg_hash_fetch_best_installation_candidate(ipkg_conf_t *conf, abstract_pkg_t *apkg, 
-						  int (*constraint_fcn)(pkg_t *pkg, void *cdata), void *cdata)
+						  int (*constraint_fcn)(pkg_t *pkg, void *cdata), void *cdata, int quiet)
 {
      int i; 
      int nprovides = 0;
@@ -185,7 +211,7 @@ pkg_t *pkg_hash_fetch_best_installation_candidate(ipkg_conf_t *conf, abstract_pk
 	       for (i = 0; i < vec->len; i++) {
 		    pkg_t *maybe = vec->pkgs[i];
 		    /* only consider the ones that satisfy the constraint */
-		    ipkg_message(conf, IPKG_DEBUG, "  %s arch=%s arch_priority=%d constraint=%d\n",
+		    ipkg_message(conf, IPKG_DEBUG, "  %s arch=%s arch_priority=%d constraint=%d \n",
 				 maybe->name, maybe->architecture,
 				 maybe->arch_priority,
 				 ((constraint_fcn == NULL) || constraint_fcn(maybe, cdata)));
@@ -218,7 +244,7 @@ pkg_t *pkg_hash_fetch_best_installation_candidate(ipkg_conf_t *conf, abstract_pk
 	  }
      }
 
-     if (!held_pkg && !latest_installed_parent && matching_apkgs->len > 1) {
+     if (!held_pkg && !latest_installed_parent && matching_apkgs->len > 1 && !quiet) {
 	  ipkg_message(conf, IPKG_ERROR, "Package=%s, %d matching providers\n",
 		       apkg->name, matching_apkgs->len);
 	  if (conf->verbosity > 1) {
@@ -240,7 +266,6 @@ pkg_t *pkg_hash_fetch_best_installation_candidate(ipkg_conf_t *conf, abstract_pk
 	  }
      }
 
- done:
      nmatching = matching_apkgs->len;
 
      pkg_vec_free(matching_pkgs);
@@ -284,7 +309,7 @@ pkg_t *pkg_hash_fetch_best_installation_candidate_by_name(ipkg_conf_t *conf, con
      if (!(apkg = abstract_pkg_fetch_by_name(hash, name)))
 	  return NULL;
 
-     return pkg_hash_fetch_best_installation_candidate(conf, apkg, pkg_name_constraint_fcn, apkg->name);
+     return pkg_hash_fetch_best_installation_candidate(conf, apkg, pkg_name_constraint_fcn, apkg->name, 0);
 }
 
 
@@ -321,14 +346,14 @@ pkg_t *pkg_hash_fetch_installed_by_name_dest(hash_table_t *hash,
     pkg_vec_t * vec;
     register int i;
 
-    if(!(vec = pkg_vec_fetch_by_name(hash, pkg_name)))
+    if(!(vec = pkg_vec_fetch_by_name(hash, pkg_name))) {
 	return NULL;
+    }
     
     for(i = 0; i < vec->len; i++)
-	if((vec->pkgs[i]->state_status == SS_INSTALLED || vec->pkgs[i]->state_status == SS_UNPACKED)
-	   && vec->pkgs[i]->dest == dest)
+	if((vec->pkgs[i]->state_status == SS_INSTALLED || vec->pkgs[i]->state_status == SS_UNPACKED) && vec->pkgs[i]->dest == dest) {
 	    return vec->pkgs[i];
-    
+        }
     return NULL;
 }
 
@@ -338,12 +363,14 @@ pkg_t *pkg_hash_fetch_installed_by_name(hash_table_t *hash,
     pkg_vec_t * vec;
     register int i;
 
-    if(!(vec = pkg_vec_fetch_by_name(hash, pkg_name)))
+    if(!(vec = pkg_vec_fetch_by_name(hash, pkg_name))){
 	return NULL;
-    
+    } 
+
     for(i = 0; i < vec->len; i++)
-	if (vec->pkgs[i]->state_status == SS_INSTALLED || vec->pkgs[i]->state_status == SS_UNPACKED)
+	if (vec->pkgs[i]->state_status == SS_INSTALLED || vec->pkgs[i]->state_status == SS_UNPACKED){
 	    return vec->pkgs[i];
+        } 
     
     return NULL;
 }
@@ -352,19 +379,19 @@ pkg_vec_t *pkg_vec_fetch_by_name(hash_table_t *hash, const char *pkg_name)
 {
     abstract_pkg_t * ab_pkg;
 
-    if(!(ab_pkg = abstract_pkg_fetch_by_name(hash, pkg_name)))
-	return NULL;
+    if(!(ab_pkg = abstract_pkg_fetch_by_name(hash, pkg_name))){
+       return NULL;
+    }
     
     if (ab_pkg->pkgs) {
       return ab_pkg->pkgs;
     } else if (ab_pkg->provided_by) {
       abstract_pkg_t *abpkg =  abstract_pkg_vec_get(ab_pkg->provided_by, 0);
-      // fprintf(stderr, __FUNCTION__ ": name=%s\n", pkg_name);
-      // fprintf(stderr, "\t first provided_by is %s\n", (abpkg ? abpkg->name : "no provided_by"));
-      if (abpkg != NULL)
+      if (abpkg != NULL){
 	  return abpkg->pkgs;
-      else
+      } else {
 	  return ab_pkg->pkgs;
+      }
     } else {
       return NULL;
     }
@@ -465,7 +492,7 @@ abstract_pkg_t * ensure_abstract_pkg_by_name(hash_table_t * hash, const char * p
   return ab_pkg;
 }
 
-pkg_t *hash_insert_pkg(hash_table_t *hash, pkg_t *pkg, int set_status)
+pkg_t *hash_insert_pkg(hash_table_t *hash, pkg_t *pkg, int set_status,ipkg_conf_t *conf)
 {
      abstract_pkg_t * ab_pkg;
      int arch_priority;
@@ -493,7 +520,7 @@ pkg_t *hash_insert_pkg(hash_table_t *hash, pkg_t *pkg, int set_status)
 	  ab_pkg->pkgs = pkg_vec_alloc();
     
      /* pkg_vec_insert_merge might munge package, but it returns an unmunged pkg */
-     pkg = pkg_vec_insert_merge(ab_pkg->pkgs, pkg, set_status);
+     pkg = pkg_vec_insert_merge(ab_pkg->pkgs, pkg, set_status,conf );
      pkg->parent = ab_pkg;
 
      if (buildProvides(hash, ab_pkg, pkg)<0){
@@ -555,7 +582,7 @@ int file_hash_set_file_owner(ipkg_conf_t *conf, const char *file_name, pkg_t *ow
 	  }
      }
 
-     ipkg_message(conf, IPKG_DEBUG2, "owning_pkg=%s filename=%s\n", owning_pkg->name, file_name);
+     // ipkg_message(conf, IPKG_DEBUG2, "owning_pkg=%s filename=%s\n", owning_pkg->name, file_name);
      hash_table_insert(file_hash, file_name, owning_pkg); 
      if (old_owning_pkg) {
 	  str_list_remove_elt(old_owning_pkg->installed_files, file_name);
